<?php

@ini_set("display_errors", 1);
error_reporting(E_ALL & ~(E_STRICT | E_NOTICE | E_DEPRECATED));

if (!defined('BASEPATH'))
    exit('No direct script access allowed');

class Install extends N_Controller {

    public $install_url = false;
    public $language = DEFAULT_LANGUAGE;
    public $lang_code = DEFAULT_LANG_CODE;
    public $service_mode = false;
    public $database_group = 'default';
    public $lang_code_prefix = DEFAULT_LANG_CODE_PREFIX;
    public $connection_id;

    public function __construct() {
        parent::__construct();
        if (!defined('MODULEPATH'))
            define('MODULEPATH', MODULESPATH . '/install/');
        if (!isset($_SESSION)) {
            @session_start();
        }
        $this->load->helper('Core');
        if (isset($this->router->lang_mode) && strlen($this->router->lang_mode) == 2) {
            switch ($this->router->lang_mode) {
                case 'ru' :
                    $this->language = 'russian';
                    $this->lang_code = 'ru';
                    break;
                case 'en' :
                    $this->language = 'english';
                    $this->lang_code = 'en';
                    break;
                default :
                    $this->language = DEFAULT_LANGUAGE;
                    $this->lang_code = DEFAULT_LANG_CODE;
            }
        }
        $this->lang_code_prefix = $this->lang_code . '/';
        if ($this->config->item('service') == TRUE) {
            $this->service_mode = TRUE;
            $this->database_group = str_replace(array(
                '.newsletterplus.net',
                '.yoursiteplus.net',
                '.jobboardplus.net'
                    ), '', $_SERVER ['SERVER_NAME']);
        }
        $this->lang->load('install', $this->language);
        $this->install_url = $this->config->item('install_url');
        $this->load->model('Install_model');
        $this->Install_model->install_url = $this->install_url;
        if (($this->config->item('installed') && $this->config->item('service') == FALSE) || $this->config->item($this->database_group)) {
            redirect(site_url($this->lang_code));
        }
        if (!isset($this->db) || !$this->db || !$this->db->table_exists('sessions')) {
            $config ['sess_use_database'] = FALSE;
            $this->load->library('session', $config);
        }
        /*
         * if ($this->config->item('installed') == TRUE && (!$this->service_mode || $this->config->item($this->database_group))) { $this->load->database($this->database_group); $this->load->library('session'); $this->load->helper('languages'); $this->load->helper('Options'); }
         */
        $this->load_css()->load_js();
        $data = array(
            'current_language' => $this->language,
            'lang_code' => $this->lang_code,
            'lang_code_prefix' => $this->lang_code . '/',
            'ajax' => is_ajax()
        );
        $this->load->vars($data);
    }

    public function get_sys_msg() {
        if (!isset($_SESSION))
            @session_start();
        $this->System_messages_model->error = isset($_SESSION ['error']) ? $_SESSION ['error'] : '';
        $this->System_messages_model->info = isset($_SESSION ['info']) ? $_SESSION ['info'] : '';
        $this->System_messages_model->success = isset($_SESSION ['success']) ? $_SESSION ['success'] : '';
        unset($_SESSION ['error']);
        unset($_SESSION ['info']);
        unset($_SESSION ['success']);
        $data = array(
            'error' => $this->System_messages_model->error,
            'info' => $this->System_messages_model->info,
            'success' => $this->System_messages_model->success
        );
        $this->load->vars($data);
        $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . "/sys_msg_refresh.php");
    }

    private function _show_main_page($data = array()) {
        $this->load->vars($data);
        $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . '/main.php');
    }

    /*
     * private function _show_progress_page($data = array()) { $this->load->vars($data); $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . '/progress.php'); }
     */

    public function index() {
        $action = $this->input->post('action', true);
        if ($action == 'language') {
            $language = $this->input->post('language', true);
            switch ($language) {
                case 'russian' :
                    redirect(site_url('ru/install/license/'));
                    break;
                default :
                    redirect(site_url('en/install/license/'));
                    break;
            }
        }
        $main_data = array();
        $data = array(
            'page_name' => 'language',
            'main_content' => $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . '/select_your_language.php', $main_data, true)
        );
        $this->_show_main_page($data);
    }

    public function license() {
        $action = $this->input->post('action', true);
        $license_key = strtoupper(format_keyword($this->input->post('license_key', true)));
        $errors = array();
        if ($action == 'license' && $license_key) {
            $result = $this->_check_license($license_key);
            if (isset($result ['errors']) && is_array($result ['errors']) && count($result) > 0) {
                foreach ($result ['errors'] as $error) {
                    $errors [] = $error;
                }
            }
            if (count($errors) > 0) {
                foreach ($errors as $error) {
                    $this->System_messages_model->add_error($error, false);
                }
            } else {
                $result = $this->_new_installation($license_key);

                // Get Modules
                $result = $this->_get_modules_info($license_key);

                if (isset($result ['errors']) && is_array($result ['errors']) && count($result) > 0) {
                    foreach ($result ['errors'] as $error) {
                        $errors [] = $error;
                    }
                }

                if (count($errors) > 0) {
                    foreach ($errors as $error) {
                        $this->System_messages_model->add_error($error, false);
                    }
                } else {
                    $result ['result'] = (array) $result ['result'];
                    $this->session->set_userdata(array(
                        'product_key' => $license_key
                    ));
                    $_SESSION['product_key'] = $license_key;
                    if ($config_handle = @fopen(FCPATH . 'application/config/nproject.php', 'r')) {
                        $config_text = @fread($config_handle, filesize(FCPATH . 'application/config/nproject.php'));
                        @fclose($config_handle);
                        $config_text = str_replace("config['product_version'] = '';", "config['product_version'] = '" . $result ['result'] ['version'] . "';", $config_text);
                        $config_text = str_replace("config['product_key'] = '';", "config['product_key'] = '" . $license_key . "';", $config_text);
                        $config_text = str_replace("config['product_key'] = '" . $this->config->item('product_key') . "';", "config['product_key'] = '" . $license_key . "';", $config_text);
                        $config_handle = @fopen(FCPATH . 'application/config/nproject.php', 'w');
                        @fwrite($config_handle, $config_text);
                        @fclose($config_handle);
                    }
                    if ($config_handle = @fopen(FCPATH . 'application/config/config.php', 'r')) {
                        $config_text = @fread($config_handle, filesize(FCPATH . 'application/config/config.php'));
                        @fclose($config_handle);
                        $base_url = $_SERVER ['HTTP_ORIGIN'];
                        if ($base_url && $base_url != '/') {
                            $config_text = str_replace("config['base_url'] = '';", "config['base_url'] = '" . $base_url . "/';", $config_text);
                        }
                        $index_page = ltrim(str_replace(array(
                            '/index.php',
                            'index.php'
                                        ), '', $_SERVER ['SCRIPT_NAME']), '/');
                        if ($index_page && $index_page != '/' && $base_url && $base_url != '/') {
                            $config_text = str_replace("config['index_page'] = '';", "config['index_page'] = '" . $index_page . "/';", $config_text);
                        }
                        $config_handle = @fopen(FCPATH . 'application/config/config.php', 'w');
                        @fwrite($config_handle, $config_text);
                        @fclose($config_handle);
                    }
                    redirect(site_url($this->lang_code_prefix . 'install/ftp'));
                }
            }
        } else {
            $this->System_messages_model->add_info($this->lang->line('encode_info'), false);
        }
        $main_data = array(
            'license' => @file_get_contents(MODULESPATH . 'install/content/licenses/' . $this->language . '.txt'),
            'license_key' => $license_key
        );
        $data = array(
            'page_name' => 'license',
            'main_content' => $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . '/license_verification.php', $main_data, true)
        );
        $this->_show_main_page($data);
    }

    public function ftp() {
        $product_key = $this->session->userdata('product_key');
        if (!$product_key) {
            $product_key = $_SESSION['product_key'];
            if (!$product_key) {
                redirect(site_url($this->lang_code_prefix . 'install/license'));
            }
        }
        $action = $this->input->post('action', true);
        $errors = array();
        $install_config_path = APPPATH . 'config/ftp' . EXT;
        $this->load->model('Install_model');
        $config_writeable = $this->Install_model->is_file_writable($install_config_path, 0777);
        $ftp_host = $this->input->post('ftp_host', true);
        $ftp_user = $this->input->post('ftp_user', true);
        $ftp_pass = $this->input->post('ftp_pass', true);
        if ($action == 'ftp') {
            if (extension_loaded('ftp') && $ftp_host) {
                if (!$ftp_user) {
                    $errors [] = $this->lang->line('ftp_empty_user');
                } else {
                    $this->connection_id = @ftp_connect($ftp_host);
                    $login_result = false;
                    if ($this->connection_id) {
                        $login_result = @ftp_login($this->connection_id, $ftp_user, $ftp_pass);
                    } else {
                        $ftp_errors[] = sprintf($this->lang->line('ftp_couldnt_connect'), $ftp_host);
                    }
                    if (!$login_result) {
                        $ftp_errors[] = sprintf($this->lang->line('ftp_connection_has_failed'), $ftp_host, $ftp_user);
                        ;
                    }
                    @ftp_close($this->connection_id);
                    if (is_array($ftp_errors) && count($ftp_errors) > 0) {
                        foreach ($ftp_errors as $error) {
                            $errors [] = $error;
                        }
                    }
                }
                if (count($errors) > 0) {
                    foreach ($errors as $error) {
                        $this->System_messages_model->add_error($error, false);
                    }
                } else {
                    $ftp_file = $install_config_path;
                    $ftp_default_file = APPPATH . 'config/ftp.default';
                    @unlink($ftp_file);
                    @copy($ftp_default_file, $ftp_file);
                    $handle_file = $ftp_file;
                    if ($config_handle = @fopen($handle_file, 'r')) {
                        $config_text = @fread($config_handle, filesize($handle_file));
                        @fclose($config_handle);
                        $config_text = str_replace('FTP_HOST', $ftp_host, $config_text);
                        $config_text = str_replace('FTP_USER', $ftp_user, $config_text);
                        $config_text = str_replace('FTP_PASS', $ftp_pass, $config_text);
                        $config_handle = @fopen($ftp_file, 'w');
                        @fwrite($config_handle, $config_text);
                        @fclose($config_handle);
                        redirect(site_url($this->lang_code_prefix . 'install/database'));
                    } else
                        $this->System_messages_model->add_error($this->lang->line('write_config_error') . ' ' . $ftp_file, false);
                }
            } else {
                redirect(site_url($this->lang_code_prefix . 'install/database'));
            }
        }
        if (!$config_writeable) {
            $this->System_messages_model->add_error($this->lang->line('ftp_permission_error'), false);
        }
        $this->config->load('ftp', true);
        $ftp_settings = $this->config->item('ftp');
        if (!$ftp_host && isset($ftp_settings ['hostname'])) {
            $ftp_host = $ftp_settings ['hostname'];
        }
        if (!$ftp_user && isset($ftp_settings ['username'])) {
            $ftp_user = $ftp_settings ['username'];
        }
        if (!$ftp_pass && isset($ftp_settings ['password'])) {
            $ftp_pass = $ftp_settings ['password'];
        }
        $main_data = array(
            'install_config_path' => $install_config_path,
            'config_writeable' => $config_writeable,
            'ftp' => $ftp,
            'ftp_host' => $ftp_host,
            'ftp_user' => $ftp_user,
            'ftp_pass' => $ftp_pass
        );
        $data = array(
            'page_name' => 'ftp',
            'main_content' => $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . '/ftp_settings.php', $main_data, true)
        );
        $this->_show_main_page($data);
    }

    public function database() {
        $product_key = $this->session->userdata('product_key');
        if (!$product_key) {
            $product_key = $_SESSION['product_key'];
            if (!$product_key) {
                redirect(site_url($this->lang_code_prefix . 'install/license'));
            }
        }
        $action = $this->input->post('action', true);
        $errors = array();
        $install_config_path = APPPATH . 'config/database' . EXT;
        $this->load->model('Install_model');
        $config_writeable = $this->Install_model->is_file_writable($install_config_path, 0777);
        $database_type = $this->input->post('database_type', true);
        $database_host = $this->input->post('database_host', true);
        $database_user = $this->input->post('database_user', true);
        $database_pass = $this->input->post('database_pass', true);
        $database_name = $this->input->post('database_name', true);
        $database_pref = $this->input->post('database_pref', true);
        //$database_host = trim(str_replace(array('tcp:',',1433'), '', $database_host));
        if ($action == 'database') {
            $db_config = array(
                'hostname' => (string) $database_host,
                'username' => (string) $database_user,
                'password' => (string) $database_pass,
                'database' => (string) $database_name,
                'dbprefix' => (string) $database_pref,
                'dbdriver' => 'mysql',
                'pconnect' => FALSE,
                'db_debug' => FALSE,
                'cache_on' => FALSE,
                'cachedir' => '',
                'char_set' => 'utf8',
                'dbcollat' => 'utf8_general_ci'
            );
            $db_connection = mysql_connect($db_config ['hostname'], $db_config ['username'], $db_config ['password']);
            $db = mysql_select_db($db_config ['database'], $db_connection);
            if (!$db) {
                $this->System_messages_model->add_error($this->lang->line('db_error'), false);
            } else {
                $database_file = $install_config_path;
                $database_default_file = APPPATH . 'config/database.default';
                /*
                 * if (!$this->service_mode) { @unlink($database_file); @copy($database_default_file, $database_file); $handle_file = $database_file; } else {
                 */
                $handle_file = $database_default_file;
                // }
                if ($this->config->item('service') == TRUE && $this->config->item($this->database_group) == TRUE) {
                    redirect(site_url($this->lang_code_prefix . 'install/setup'));
                } else {
                    if ($config_handle = @fopen($handle_file, 'r')) {
                        $config_text = @fread($config_handle, filesize($handle_file));
                        @fclose($config_handle);
                        $config_text = str_replace('DB_HOST', isset($db_config ['hostname']) ? $db_config ['hostname'] : '', $config_text);
                        $config_text = str_replace('DB_USER', isset($db_config ['username']) ? $db_config ['username'] : '', $config_text);
                        $config_text = str_replace('DB_PASSWORD', isset($db_config ['password']) ? $db_config ['password'] : '', $config_text);
                        $config_text = str_replace('DB_NAME', isset($db_config ['database']) ? $db_config ['database'] : '', $config_text);
                        $config_text = str_replace('DB_PREFIX', isset($db_config ['dbprefix']) ? $db_config ['dbprefix'] : '', $config_text);
                        /* if ($this->service_mode) { */
                        $database_handle = @fopen($database_file, 'r');
                        $database_text = @fread($database_handle, filesize($database_file));
                        $config_text = $database_text . str_replace('default', $this->database_group, $config_text);
                        // }
                        $config_handle = @fopen($database_file, 'w');
                        @fwrite($config_handle, $config_text);
                        @fclose($config_handle);
                        if ($this->database_group != 'service') {
                            if ($config_handle = @fopen(APPPATH . 'config/nproject' . EXT, 'r')) {
                                $config_text = @fread($config_handle, filesize(APPPATH . 'config/nproject' . EXT));
                                @fclose($config_handle);
                                $config_text .= "\$config['" . $this->database_group . "'] = FALSE;\n\r";
                                $config_handle = @fopen(APPPATH . 'config/nproject' . EXT, 'w');
                                @fwrite($config_handle, $config_text);
                                @fclose($config_handle);
                            }
                        }
                        $result = $this->_get_modules_info($product_key);

                        if (isset($result ['errors']) && is_array($result ['errors']) && count($result) > 0) {
                            foreach ($result ['errors'] as $error) {
                                $errors [] = $error;
                            }
                        }
                        if (count($errors) > 0) {
                            foreach ($errors as $error) {
                                $this->System_messages_model->add_error($error, false);
                            }
                        } else {
                            $result ['result'] = (array) $result ['result'];
                            $modules = array();
                            if (isset($result ['result'] ['modules'])) {
                                foreach ($result ['result'] ['modules'] as $module) {
                                    $module = (array) $module;
                                    $modules [] = $module;
                                }
                            }
                            $this->load->helper('Modules');
                            $this->load->database($this->database_group);
                            if (function_exists('create_modules_table')) {
                                create_modules_table();
                            }
                            foreach ($modules as $module) {
                                if (in_array($module ['keyword'], array(
                                            'options',
                                            'languages',
                                        ))) {
                                    if (function_exists('delete_module_by_keyword')) {
                                        delete_module_by_keyword($module ['keyword']);
                                    }
                                    if (function_exists('save_module')) {
                                        $module_data = array(
                                            'keyword' => $module ['keyword'],
                                            'module_type' => $module ['module_type'],
                                            'name' => $module ['name'],
                                            'description' => $module ['description'],
                                            'module_version' => $module ['module_version'],
                                            'sub_version' => $module ['sub_version'],
                                            'patch_version' => $module ['patch_version'],
                                            'status' => 1
                                        );
                                        save_module($module_data);
                                    }
                                }
                            }
                            foreach ($modules as $module) {
                                if (in_array($module ['keyword'], array(
                                            'services'
                                        ))) {
                                    if (function_exists('delete_module_by_keyword')) {
                                        delete_module_by_keyword($module ['keyword']);
                                    }
                                    if (function_exists('save_module')) {
                                        $module_data = array(
                                            'keyword' => $module ['keyword'],
                                            'module_type' => $module ['module_type'],
                                            'name' => $module ['name'],
                                            'description' => $module ['description'],
                                            'module_version' => $module ['module_version'],
                                            'sub_version' => $module ['sub_version'],
                                            'patch_version' => $module ['patch_version'],
                                            'status' => 1
                                        );
                                        save_module($module_data);
                                    }
                                }
                            }
                            foreach ($modules as $module) {
                                if (!in_array($module ['keyword'], array(
                                            'options',
                                            'languages',
                                            'services',
                                            'fields_editor'
                                        ))) {
                                    if (function_exists('delete_module_by_keyword')) {
                                        delete_module_by_keyword($module ['keyword']);
                                    }
                                    if (function_exists('save_module')) {
                                        $module_data = array(
                                            'keyword' => $module ['keyword'],
                                            'module_type' => $module ['module_type'],
                                            'name' => $module ['name'],
                                            'description' => $module ['description'],
                                            'module_version' => $module ['module_version'],
                                            'sub_version' => $module ['sub_version'],
                                            'patch_version' => $module ['patch_version'],
                                            'status' => 1
                                        );
                                        save_module($module_data);
                                    }
                                }
                            }
                            foreach ($modules as $module) {
                                if (in_array($module ['keyword'], array(
                                            'fields_editor'
                                        ))) {
                                    if (function_exists('delete_module_by_keyword')) {
                                        delete_module_by_keyword($module ['keyword']);
                                    }
                                    if (function_exists('save_module')) {
                                        $module_data = array(
                                            'keyword' => $module ['keyword'],
                                            'module_type' => $module ['module_type'],
                                            'name' => $module ['name'],
                                            'description' => $module ['description'],
                                            'module_version' => $module ['module_version'],
                                            'sub_version' => $module ['sub_version'],
                                            'patch_version' => $module ['patch_version'],
                                            'status' => 1
                                        );
                                        save_module($module_data);
                                    }
                                }
                            }
                            redirect(site_url($this->lang_code_prefix . 'install/download'));
                        }
                    } else
                        $this->System_messages_model->add_error($this->lang->line('write_config_error') . ' ' . FCPATH . 'application/config/database.php', false);
                }
            }
        }
        @require_once (APPPATH . 'config/database' . EXT);
        if (isset($db [$this->database_group])) {
            $database_settings = $db [$this->database_group];
            if (!$database_host && isset($database_settings ['hostname'])) {
                $database_host = $database_settings ['hostname'];
            }
            if (!$database_user && isset($database_settings ['username'])) {
                $database_user = $database_settings ['username'];
            }
            if (!$database_pass && isset($database_settings ['password'])) {
                $database_pass = $database_settings ['password'];
            }
            if (!$database_name && isset($database_settings ['database'])) {
                $database_name = $database_settings ['database'];
            }
            if (!$database_pref && isset($database_settings ['dbprefix'])) {
                $database_pref = $database_settings ['dbprefix'];
            }
        }
        $main_data = array(
            'install_config_path' => $install_config_path,
            'config_writeable' => $config_writeable,
            'database_host' => $database_host,
            'database_user' => $database_user,
            'database_pass' => $database_pass,
            'database_name' => $database_name,
            'database_pref' => $database_pref
        );
        $data = array(
            'page_name' => 'database',
            'main_content' => $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . '/database_settings.php', $main_data, true)
        );
        $this->_show_main_page($data);
    }

    protected function _change_permissions($orig_path = '', $path = '', $mode = 0777, $files = array()) {
        $scan = scandir($path);
        if ($orig_path != $path) {
            $path_prefix = '/';
        } else {
            $path_prefix = '';
        }
        if (is_array($scan) && count($scan) > 0) {
            foreach ($scan as $file) {
                if (!in_array($file, array(
                            '.',
                            '..',
                            'nps',
                            'ysps'
                        ))) {
                    if (is_dir($path . $path_prefix . $file)) {
                        $writeable = $this->Install_model->is_file_writable(str_replace($orig_path, '', $path . $path_prefix . $file), $mode);
                        if (!$writeable) {
                            $files [] = str_replace($orig_path, '', $path . $path_prefix . $file);
                        }
                        $files = $this->_change_permissions($orig_path, $path . $path_prefix . $file, $mode, $files);
                    } else {
                        $writeable = $this->Install_model->is_file_writable(str_replace($orig_path, '', $path . $path_prefix . $file), $mode);
                        if (!$writeable) {
                            $files [] = str_replace($orig_path, '', $path . $path_prefix . $file);
                        }
                    }
                }
            }
        }
        return $files;
    }

    public function download($module_id = -1) {
        if ($this->service_mode) {
            redirect(site_url($this->lang_code_prefix . 'install/setup'));
        }
        $product_key = $this->session->userdata('product_key');
        if (!$product_key) {
            $product_key = $_SESSION['product_key'];
            if (!$product_key) {
                redirect(site_url($this->lang_code_prefix . 'install/license'));
            }
        }
        $this->load->helper('Core');
        $this->load->database($this->database_group);
        $this->db->cache_delete_all();
        $this->load->helper('Modules');
        if (function_exists('get_modules_list')) {
            $modules = get_modules_list(null, null, array('id' => 'asc'));
        } else {
            $modules = array();
        }
        if (!is_array($modules) || count($modules) == 0 || !$this->config->item('product_key') || !$this->config->item('product_version')) {
            redirect(site_url($this->lang_code_prefix . 'install/license'));
        }
        $action = $this->input->post('action', true);
        $errors = array();
        $this->load->model('Install_model');
        $files = array();
        if ($module_id < 0) {
            $files = $this->_change_permissions(FCPATH, APPPATH, 0777, $files);
            $files = $this->_change_permissions(FCPATH, UPLOADSPATH, 0777, $files);
            $files = $this->_change_permissions(FCPATH, TEMPPATH, 0777, $files);
        }
        if ($action == 'download' || $module_id >= 0) {
            if (is_array($files) && count($files) > 0) {
                $errors [] = $this->lang->line('download_permissions_error');
            }
            if (count($errors) > 0) {
                foreach ($errors as $error) {
                    $this->System_messages_model->add_error($error, false);
                }
            } else {
                if ($module_id < 0) {
                    $module_id = 0;
                }
                if (isset($modules [$module_id])) {
                    if (!$this->config->item('service')) {
                        $module = $modules [$module_id];
                        $result = $this->_download_module($this->config->item('product_key'), $module ['keyword'], true);
                        if ($result) {
                            $file_content = $result;
                            $file_handle = @fopen(TEMPPATH . 'downloads/' . $module ['keyword'] . '.zip', 'w');
                            @fwrite($file_handle, $file_content);
                            @fclose($file_handle);
                            if (class_exists('ZipArchive')) {
                                $zip = new ZipArchive;
                                $res = $zip->open(FCPATH . 'temp/downloads/' . $module ['keyword'] . '.zip');
                                if ($res === TRUE) {
                                    $zip->extractTo(FCPATH);
                                    $zip->close();
                                }
                            } else {
                                $this->load->library('Pclzip', TEMPPATH . 'downloads/' . $module ['keyword'] . '.zip');
                                $this->pclzip->extract('./');
                            }
                            // @unlink ( TEMPPATH . 'downloads/' . $module ['keyword'] . '.zip' );
                            if (!is_file(MODULESPATH . $module ['keyword'] . '/models/' . $module ['keyword'] . '_install_model.php')) {
                                show_error('download', 500);
                            }
                        }
                    }
                } else {
                    $module_id = 999999999;
                }
            }
        }
        $main_data = array(
            'files' => $files,
            'modules' => $modules,
            'module_id' => $module_id,
            'action' => $action
        );
        $data = array(
            'page_name' => 'download',
            'main_content' => $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . '/download_modules.php', $main_data, true)
        );
        $this->_show_main_page($data);
    }

    public function setup($module_id = -1) {
        $product_key = $this->session->userdata('product_key');
        if (!$product_key) {
            $product_key = $_SESSION['product_key'];
            if (!$product_key) {
                redirect(site_url($this->lang_code_prefix . 'install/license'));
            }
        }
        $this->load->helper('Core');
        $this->load->database($this->database_group);
        $this->db->cache_delete_all();
        if ($this->db->table_exists('sessions')) {
            $this->load->library('session');
        }
        $this->load->helper('Modules');
        if (function_exists('get_modules_list')) {
            $modules = get_modules_list(null, null, array('id' => 'asc'));
        } else {
            $modules = array();
        }
        if (!is_array($modules) || count($modules) == 0 || !$this->config->item('product_key') || !$this->config->item('product_version')) {
            redirect(site_url($this->lang_code_prefix . 'install/license'));
        }
        $action = $this->input->post('action', true);
        $errors = array();
        $this->load->model('Install_model');
        $files = array();
        if (!$this->service_mode && $module_id < 0) {
            $files = $this->_change_permissions(FCPATH, APPPATH, 0777, $files);
            $files = $this->_change_permissions(FCPATH, UPLOADSPATH, 0777, $files);
            $files = $this->_change_permissions(FCPATH, TEMPPATH, 0777, $files);
        }
        if ($action == 'setup' || $module_id >= 0) {
            if (is_array($files) && count($files) > 0) {
                $errors [] = $this->lang->line('setup_permissions_error');
            }
            if (count($errors) > 0) {
                foreach ($errors as $error) {
                    $this->System_messages_model->add_error($error, false);
                }
            } else {
                if ($module_id < 0) {
                    $module_id = 0;
                }
                if (isset($modules [$module_id])) {
                    $module = $modules [$module_id];
                    $model_name = $module ['keyword'] . '_install_model';
                    if (file_exists(MODULESPATH . $module ['keyword'] . '/models/' . $model_name . '.php')) {
                        $this->load->model($model_name);
                        if (method_exists($this->$model_name, 'install'))
                            $this->$model_name->install();
                    }
                } else {
                    $module_id = 999999999;
                }
            }
        }
        $main_data = array(
            'files' => $files,
            'modules' => $modules,
            'module_id' => $module_id,
            'action' => $action
        );
        $data = array(
            'page_name' => 'setup',
            'main_content' => $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . '/setup_modules.php', $main_data, true)
        );
        $this->_show_main_page($data);
    }

    public function complete() {
        $product_key = $this->session->userdata('product_key');
        if (!$product_key) {
            $product_key = $_SESSION['product_key'];
            if (!$product_key) {
                redirect(site_url($this->lang_code_prefix . 'install/license'));
            }
        }
        $this->load->helper('Core');
        $this->load->database($this->database_group);
        $this->db->cache_delete_all();
        $this->load->helper('Modules');
        if (function_exists('get_modules_list')) {
            $modules = get_modules_list(null, null, array('id' => 'asc'));
        } else {
            $modules = array();
        }
        if (!is_array($modules) || count($modules) == 0 || !$this->config->item('product_key') || !$this->config->item('product_version')) {
            redirect(site_url($this->lang_code_prefix . 'install/license'));
        }
        $action = $this->input->post('action', true);
        $email = $this->input->post('email', true);
        $password = $this->input->post('password', true);
        $fname = $this->input->post('fname', true);
        $sname = $this->input->post('sname', true);
        $errors = array();
        $this->load->model('Install_model');
        $files = array();
        if (!$this->service_mode && !$action) {
            $files = $this->_change_permissions(FCPATH, APPPATH, 0777, $files);
            $files = $this->_change_permissions(FCPATH, UPLOADSPATH, 0777, $files);
            $files = $this->_change_permissions(FCPATH, TEMPPATH, 0777, $files);
        }
        if ($action == 'complete') {
            if (is_array($files) && count($files) > 0) {
                $errors [] = $this->lang->line('setup_permissions_error');
            }
            if (!$email) {
                $errors [] = $this->lang->line('complete_email_error');
            }
            if (!$password) {
                $errors [] = $this->lang->line('complete_password_error');
            }
            if (count($errors) > 0) {
                foreach ($errors as $error) {
                    $this->System_messages_model->add_error($error, false);
                }
            } else {
                $this->load->helper('Users');
                $account = false;
                if (function_exists('get_account_by_email')) {
                    $account = get_account_by_email($email);
                }
                if ($account && function_exists('delete_account')) {
                    delete_account($account);
                    if (function_exists('get_profile_by_user')) {
                        $profile = get_profile_by_user($account->id);
                        if ($profile && function_exists('delete_profile')) {
                            delete_profile($profile);
                        }
                    }
                }
                $account = array(
                    'account_type' => 'admin',
                    'email' => $email,
                    'password' => md5($password),
                    'language' => $this->language,
                    'status' => 1,
                    'confirm' => 1,
                    'date_created' => time()
                );
                if (function_exists('save_user_account')) {
                    $account_id = save_user_account($account);
                    $profile = array(
                        'user_id' => $account_id,
                        'fname' => $fname,
                        'sname' => $sname,
                        'date_created' => time()
                    );
                    if (function_exists('save_user_profile')) {
                        $profile_id = save_user_profile($profile);
                    }
                    $this->load->library('session');
                    $this->session->set_userdata('id', $account_id);
                    $this->session->set_userdata('usertype', 'admin');
                    $this->session->set_userdata('logged_in', TRUE);
                    $this->session->set_userdata('language', $this->language);
                }
                if ($config_handle = @fopen(FCPATH . 'application/config/nproject.php', 'r')) {
                    $config_text = @fread($config_handle, filesize(FCPATH . 'application/config/nproject.php'));
                    @fclose($config_handle);
                    $config_text = str_replace("config['installed'] = FALSE;", "config['installed'] = TRUE;", $config_text);
                    if ($this->config->item('service') == TRUE) {
                        $config_text = str_replace("config['" . $this->database_group . "'] = FALSE;", "config['" . $this->database_group . "'] = TRUE;", $config_text);
                    }
                    $config_handle = @fopen(FCPATH . 'application/config/nproject.php', 'w');
                    @fwrite($config_handle, $config_text);
                    @fclose($config_handle);
                }
                $this->db->cache_delete_all();
                redirect(site_url($this->lang_code_prefix . 'admin/'));
            }
        }
        $main_data = array(
            'email' => $email,
            'password' => $password,
            'fname' => $fname,
            'sname' => $sname,
            'files' => $files
        );
        $data = array(
            'page_name' => 'complete',
            'main_content' => $this->parser->parse(MODULESPATH . 'install/views/' . $this->theme . '/complete.php', $main_data, true)
        );
        $this->_show_main_page($data);
    }

    protected function load_js() {
        $data = array(
            'js' => array(
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.min.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/bootstrap/bootstrap.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/ios-orientationchange-fix.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery-ui.min.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.ui.touch-punch.min.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.validate.min.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.uniform.min.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.qtip.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.pnotify.min.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.ibutton.min.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.dataTables.min.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.cookie.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.fancybox.pack.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/jquery/jquery.ui.totop.min.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/responsive-tables.js'),
                site_url('uploads/modules/ckeditor/ckeditor.js'),
                site_url('uploads/modules/ckeditor/adapters/jquery.js'),
                site_url('uploads/modules/main/' . $this->theme . '/js/core.js')
            )
        );

        $this->load->vars($data);
        return $this;
    }

    protected function load_css() {
        $data = array(
            'css' => array(
                site_url('uploads/modules/main/' . $this->theme . '/css/bootstrap/bootstrap.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/bootstrap/bootstrap-responsive.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/supr-theme/jquery.ui.supr.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/icons.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/uniform/uniform.default.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/main.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/jquery/jquery.qtip.min.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/jquery/jquery.dataTables.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/jquery/jquery.pnotify.default.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/jquery/jquery.ibutton.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/jquery/jquery.fancybox.css'),
                site_url('uploads/modules/main/' . $this->theme . '/css/responsive-tables.css')
            )
        );
        $this->load->vars($data);
        return $this;
    }

    public function files() {
        if (!defined('MODULEPATH'))
            exit();
        $args = func_get_args();
        $file_path = implode('/', $args);
        if (empty($file_path)) {
            exit();
        }
        $file_path = MODULEPATH . 'content/' . $file_path;
        if (file_exists($file_path)) {

            if ($fd = @fopen($file_path, "r")) {
                $fsize = @filesize($file_path);
                $path_parts = @pathinfo($file_path);
                if (isset($path_parts ['extension'])) {
                    $ext = strtolower($path_parts ["extension"]);
                    $file_type = function_exists('get_file_extension') ? get_file_extension($file_path) : false;
                    $expires = 60 * 60 * 24 * 14;
                    header("Last-Modified: " . date("D, d M Y H:i:s", filemtime($file_path)) . " GMT");
                    header("Cache-Control: maxage=" . $expires);
                    header('Expires: ' . gmdate('D, d M Y H:i:s', time() + $expires) . ' GMT');
                    if ($file_type)
                        header('Content-type: ' . $file_type);
                    header("Content-Disposition: filename=\"" . $path_parts ["basename"] . "\"");
                    header("Pragma: public");
                    $compressed = FALSE;
                    if ($this->config->item('compress_output') === TRUE) {
                        if (extension_loaded('zlib')) {
                            if (isset($_SERVER ['HTTP_ACCEPT_ENCODING']) and strpos($_SERVER ['HTTP_ACCEPT_ENCODING'], 'gzip') !== FALSE) {
                                $compressed = TRUE;
                                header("Content-Encoding: gzip");
                                ob_start('ob_gzhandler');
                            }
                        }
                    }
                    if (!$compressed)
                        header("Content-length: " . $fsize);
                    $buffer = @file_get_contents($file_path);
                    $buffer;
                    if (ob_get_length() > 0)
                        ob_end_flush();
                }
            }
            @fclose($fd);
            exit();
        }
    }

}
